/******************************************************************************/
// dkcrtmnu.c - Dungeon Keeper Creature editor
/******************************************************************************/
// Author:  Jon Skeet
// Created: 14 Nov 1997

// Purpose:
//   DK Creature.txt edit menu.

// Comment:
//   None.

//Copying and copyrights:
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
/******************************************************************************/

#include "dkcrtmnu.h"

#include "globals.h"

static char *ROOMS[]={"None", "Portal", "Treasure", "Library", "Prison",
    "Torture", "Training", "Heart", "Workshop", "Scavenger",
    "Temple", "Graveyard", "Barracks", "Hatchery", "Lair",
    "Bridge", "Guard post", NULL};
static const int ROOMS_LEN=(sizeof(ROOMS)/sizeof(char *))-1;

static char *CREATURES[]={"None", "Wizard", "Barbarian",
    "Archer", "Monk", "Dwarf", "Knight", "Avatar", "Tunneller",
    "Witch", "Giant", "Fairy", "Thief", "Samurai", "Horned Reaper",
    "Skeleton", "Troll", "Dragon", "Demon Spawn", "Fly", "Dark Mistress",
    "Warlock", "Bile Demon", "Imp", "Beetle", "Vampire", "Spider",
    "Hell Hound", "Ghost", "Tentacle", "Orc", "Floating spirit", NULL};
static const int CREATURES_LEN=(sizeof(CREATURES)/sizeof(char *))-1;

static char *POWERS[]={"Hand of Evil", "Create imp", "Must obey",
    "Slap", "Evil Sight", "Call to arms", "Cave in", "Heal creature",
    "Hold audience", "Lightning", "Speed", "Protect", "Conceal",
    "Disease", "Chicken", "Destroy walls", "Time bomb", "Possess",
    "Armageddon", NULL};
static const int POWERS_LEN=(sizeof(POWERS)/sizeof(char *))-1;

static char *SHOTS[]={"Fireball", "Firebomb", "Freeze", "Lightning",
    "Poison cloud", "Nav. Missile", "Flame breathe", "Wind", "Missile",
    "Slow", "Grenade", "Drain", "Hail storm", "Arrow", "Boulder",
    "God lightning", "Spike", "Vortex", "Alarm", "Solid boulder",
    "Swing sword", "Swing fist", "Dig", "Lightning ball", "Group",
    "Disease", "Chicken", "Time bomb", "Trap Lightning", NULL};
static const int SHOTS_LEN=(sizeof(SHOTS)/sizeof(char *))-1;

static char *INSTANCE[]={"None", "Swing sword", "Swing fist", 
    "Escape", "Fire arrow", "Fireball", "Fire bomb", "Freeze", 
    "Armour", "Lightning", "Rebound", "Heal", "Poision cloud", 
    "Invisibility", "Teleport", "Speed", "Slow", "Drain", "Fear", 
    "Missile", "Nav. Missile", "Flame breath", "Wind", "Light", 
    "Fly", "Sight", "Grenade", "Hailstorm", "Word of Power", 
    "Fart", "Dig", "Pretty path", "Destroy area", "Tunnel", 
    "Wave", "Reinforce", "Eat", "Attack slab", "Damage wall", 
    "1st person dig", "Cast group", "Cast disease", "Cast chicken", 
    "Cast time bomb", "Moan", "Tortured", NULL};
static const int INSTANCE_LEN=(sizeof(INSTANCE)/sizeof(char *))-1;

static char *SPELLS[]={"Fireball", "Fire bomb", "Freeze",
    "Armour", "Lightning", "Rebound", "Heal", "Poison cloud",
    "Invisibility", "Teleport", "Speed", "Slow", "Drain", "Fear",
    "Missile", "Nav. Missile", "Flame breath", "Wind", "Light",
    "Fly", "Sight", "Grenade", "Hailstorm", "Word of Power",
    "Crazy gas", "Disease", "Chicken", "Time bomb", NULL};
static const int SPELLS_LEN=(sizeof(SPELLS)/sizeof(char *))-1;

static char *JOBS[]={"None", "Tunnel", "Dig", "Research", "Train",
    "Manufacture", "Scavenge", "Kinky torture", "Fight", "Seek enemy",
    "Guard", "Group", "Barrack", "Temple", "Freeze prisoners", "Explore",
    NULL};
static const int JOBS_LEN=(sizeof(JOBS)/sizeof(char *))-1;

static char *ALIGN_JOBS[]={"None            ", "Tunnel          ", 
    "Dig             ", "Research        ", "Train           ", 
    "Manufacture     ", "Scavenge        ", "Kinky torture   ", 
    "Fight           ", "Seek enemy      ", "Guard           ", 
    "Group           ", "Barrack         ", "Temple          ", 
    "Freeze prisoners", "Explore         ",
    NULL};
static const int ALIGN_JOBS_LEN=(sizeof(ALIGN_JOBS)/sizeof(char *))-1;


static char *ANGER_JOBS[]={"Kill creatures", "Destroy rooms",
    "Leave dungeon", "Steal gold", "Damage walls", "Mad psycho", "Persuade",
    "Join enemy", NULL};
static const int ANGER_JOBS_LEN=(sizeof(ANGER_JOBS)/sizeof(char *))-1;

static char *SLABS[]={"None", "Gold", "Rock", "None", "None", "None",
    "None", "None", "None", "None", "None", "Path", "Lava", "Water"};
static const int SLABS_LEN=(sizeof(SLABS)/sizeof(char *))-1;

static char *ATTACKS[]={"None", "Melee", "Ranged", NULL};
static const int ATTACKS_LEN=(sizeof(ATTACKS)/sizeof(char *))-1;

static char *EYES[]={"Normal", "Wibble", "Fish eye", "Compound",
    "Water", "Double Water", "Dark Water", "Blob", "2xSmoke",
    "Edge smoke", "2xshit", "Quake", "Smoke", "Dog", "Vampire", NULL};
static const int EYES_LEN=(sizeof(EYES)/sizeof(char *))-1;

static char *BASIC_STATS[]={"Health", "Strength", "Armour", "Dexterity", 
    "Fear", "Defence", "Luck", "Pay", "Recovery", "Visual Range", 
    "Max Angle Change", "Eye Height", "Size XY", "Size YZ", 
    "Flying", "Thing Size XY", "Thing Size YZ", "Hearing", 
    "Bleeds", "Affected By Wind", "Can See Invisible", 
    "Through Doors", "Humanoid", 
    "Piss On Dead", "Hurt By Lava", "Base Speed", "Gold Hold", 
    "Walk Anim Speed", "Immune to Gas", "Usual Attack", 
    "Field Of View", "Eye Effect", "Rebirth", "Heal Requirment", 
    "Heal Threshold", "Hero Vs Keeper Cost", "Real Training", 
    "Torture Time", "Slaps To Kill", "Creature Loyalty", 
    "Loyalty Level", "Entrance Force", "Damage To Boulder"};
static const int BASIC_STATS_LEN=(sizeof(BASIC_STATS)/sizeof(char *))-1;

static int STAT_OFF[]={5, 8, 9, 10, 11, 12, 13, 75, 14, 29,
    43, 44, 45, 46, 47, 117, 118, 78, 120, 122, 113, 114, 73,
    74, 19, 24, 28, 48, 49, 50, 51, 110, 112, 6, 7, 76, 103,
    104, 106, 107, 108, 111, 116};

static int STAT_TYPE[]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    1,0,0,0,1,1,1,1,1,1,1,0,0,0,1,2,0,2,1,0,
    0,0,0,0,0,0,0,0,0};

static void *STAT_PARAM[]={NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
    NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
    NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
    ATTACKS, NULL, EYES, NULL, NULL, NULL, NULL, NULL,
    NULL,NULL,NULL,NULL,NULL,NULL};

static char *LIVING[]={"Primary job", "Secondary job", 
    "Sleep experience slab", "Sleep experience", 
    "Hitting experience", "Scavenge value", "Scavenge requirement",
    "Scavenge cost", "Training value", "Training cost", "Hunger rate",
    "Hunger fill", "Lair size", "Manufacture value", "Research value",
    "Lair enemy", "Primary entrance", "Secondary entrance", 
    "Tertiary entrance", "Primary slabs", "Secondary slabs", "Tertiary slabs"};
static const int LIVING_LEN=(sizeof(LIVING)/sizeof(char *))-1;

static int LIVING_OFF[]={1, 2, 20, 21, 25, 22, 23,
    30, 26, 27, 15, 16, 18, 77, 72, 109, 79, 80, 81, 82, 83, 84};

static int LIVING_TYPE[]={2,2,2,0,
    0,0,0,0,0,0,0,0,0,0,0,
    2,2,2,2,0,0,0};

static void *LIVING_PARAM[]={JOBS, JOBS, SLABS, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    CREATURES, ROOMS, ROOMS, ROOMS, NULL, NULL, NULL};
    
static void *ANNOY[]={"In Hand", "Eat Food", "Will Not Do Job", 
    "No Lair", "No Hatchery", "Woken Up", "Standing On Dead Enemy", 
    "Sulking", "No Salary", "Slapped", "Standing On Dead Friend", 
    "In Torture", "In Temple", "Sleeping", "Got Wage", "Win Battle", 
    "Untrained Time", "Untrained", "Job Stress", "Annoyance Job Stress",
    "Queue", "Others Leaving", "Annoy Level"};
static const int ANNOY_LEN=(sizeof(ANNOY)/sizeof(char *))-1;

static int ANNOY_OFF[]={121, 119, 105, 85, 86, 87, 88,
    89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 17, 115};

/*
 * Creates "Creatures" menu elements.
 */
menu init_creatures_menu(menu root,struct dk_creature_data *crdata)
{
    static int lx[3]={0,23,47};
    static int sx[3]={16, 40, 66};
    menu creatures;
    creatures = new_menu("Creature variables");
    char temp[LINEMSG_SIZE];
    int i;
    for (i=0; CREATURES[i+1]!=NULL; i++)
    {
      menu cstat, cliving, cannoy, clevel;
      // Create a menu for this creature
      sprintf (temp, "Basic stats for %s", CREATURES[i+1]);
      cstat = new_menu(strdup(temp));
      sprintf (temp, "Annoyance levels for %s", CREATURES[i+1]);
      cannoy = new_menu(strdup(temp));
      sprintf (temp, "Level stats for %s", CREATURES[i+1]);
      clevel = new_menu(strdup(temp));
      sprintf (temp, "Daily life for %s", CREATURES[i+1]);
      cliving = new_menu(strdup(temp));
    
      add_item_to_menu(creatures, menu_new_item((i>>4)*40+2, (i&15)+2, CREATURES[i+1], MIT_MENU, menu_value_longptr(NULL), cstat));

      add_item_to_menu(cstat, menu_new_item(0, 23, "Daily life", MIT_MENU, menu_value_longptr(NULL), cliving));
      add_item_to_menu(cstat, menu_new_item(26, 23, "Level stats", MIT_MENU, menu_value_longptr(NULL), clevel));
      add_item_to_menu(cstat, menu_new_item(52, 23, "Annoyance levels", MIT_MENU, menu_value_longptr(NULL), cannoy));
    
      add_item_to_menu(cliving, menu_new_item(0, 23, "Basic stats", MIT_MENU, menu_value_longptr(NULL), cstat));
      add_item_to_menu(cliving, menu_new_item(26, 23, "Level stats", MIT_MENU, menu_value_longptr(NULL), clevel));
      add_item_to_menu(cliving, menu_new_item(52, 23, "Annoyance levels", MIT_MENU, menu_value_longptr(NULL), cannoy));

      add_item_to_menu(clevel, menu_new_item(0, 23, "Basic stats", MIT_MENU, menu_value_longptr(NULL), cstat));
      add_item_to_menu(clevel, menu_new_item(26, 23, "Daily life", MIT_MENU, menu_value_longptr(NULL), cliving));
      add_item_to_menu(clevel, menu_new_item(52, 23, "Annoyance levels", MIT_MENU, menu_value_longptr(NULL), cannoy));

      add_item_to_menu(cannoy, menu_new_item(0, 23, "Basic stats", MIT_MENU, menu_value_longptr(NULL), cstat));
      add_item_to_menu(cannoy, menu_new_item(26, 23, "Daily life", MIT_MENU, menu_value_longptr(NULL), cliving));
      add_item_to_menu(cannoy, menu_new_item(52, 23, "Level stats", MIT_MENU, menu_value_longptr(NULL), clevel));

      int j;
      
      for (j=0; j < 43; j++)
      {
        add_item_to_menu(cstat, menu_new_item(lx[j/15], (j%15)+2, BASIC_STATS[j], MIT_STATIC, menu_value_longptr(NULL), NULL));
        add_item_to_menu(cstat, menu_new_item(sx[j/15], (j%15)+2, "", STAT_TYPE[j], menu_value_longptr(crdata->values+i*123+STAT_OFF[j]), STAT_PARAM[j]));
      }

      for (j=0; j < 23; j++)
      {
        add_item_to_menu(cannoy, menu_new_item((j/12)*40, (j%12)+2, ANNOY[j], MIT_STATIC, menu_value_longptr(NULL), NULL));
        add_item_to_menu(cannoy, menu_new_item((j/12)*40+30, (j%12)+2, "", MIT_NUMERIC, menu_value_longptr(crdata->values+i*123+ANNOY_OFF[j]), NULL));
      }
      add_item_to_menu(cannoy, menu_new_item(0, 15, "Behaviour when angry", MIT_STATIC, menu_value_longptr(NULL), NULL));
      for (j=0; j < 8; j++)
      {
        add_item_to_menu(cannoy, menu_new_item((j/4)*40, (j%4)+17, ANGER_JOBS[j], MIT_STATIC, menu_value_longptr(NULL), NULL));
        add_item_to_menu(cannoy, menu_new_item((j/4)*40+30, (j%4)+17, "", MIT_BOOLEAN, menu_value_longptr(crdata->jobanger+i*8+j), NULL));
      }

      for (j=0; j < 22; j++)
      {
        add_item_to_menu(cliving, menu_new_item((j/11)*40, (j%11)+2, LIVING[j], MIT_STATIC, menu_value_longptr(NULL), NULL));
        add_item_to_menu(cliving, menu_new_item((j/11)*40+22, (j%11)+2, "", LIVING_TYPE[j], menu_value_longptr(crdata->values+i*123+LIVING_OFF[j]), LIVING_PARAM[j]));
      }
    
      add_item_to_menu(cliving, menu_new_item(0, 14, "\"Not do\" jobs", MIT_STATIC, menu_value_longptr(NULL), NULL));
      for (j=0; j < 15; j++)
        add_item_to_menu(cliving, menu_new_item((j/5)*26, (j%5)+16, ALIGN_JOBS[j+1], MIT_BOOLEAN, menu_value_longptr(crdata->jobnotdo+i*15+j), NULL));
    
      add_item_to_menu(clevel, menu_new_item(0, 2, "Level", MIT_STATIC, menu_value_longptr(NULL), NULL));
      add_item_to_menu(clevel, menu_new_item(5, 2, "Exp", MIT_STATIC, menu_value_longptr(NULL), NULL));
      for (j=0; j < 10; j++)
      {
        if (j < 9)
        sprintf (temp, "%d", j+2);
        else
        sprintf (temp, "Grow");
        add_item_to_menu(clevel, menu_new_item(0, j+4, strdup(temp), MIT_STATIC, menu_value_longptr(NULL), NULL));
        sprintf (temp, "Power %d", j+1);
        add_item_to_menu(clevel, menu_new_item(15, j+4, strdup (temp), MIT_STATIC, menu_value_longptr(NULL), NULL));
        sprintf (temp, "Level req. for power %d", j+1);
        add_item_to_menu(clevel, menu_new_item(40, j+4, strdup (temp), MIT_STATIC, menu_value_longptr(NULL), NULL));
        add_item_to_menu(clevel, menu_new_item(6, j+4, "", MIT_NUMERIC, menu_value_longptr(crdata->values+i*123+31+j), NULL));
        add_item_to_menu(clevel, menu_new_item(25, j+4, "", MIT_CHOICE, menu_value_longptr(crdata->values+i*123+52+j), INSTANCE));
        add_item_to_menu(clevel, menu_new_item(65, j+4, "", MIT_NUMERIC, menu_value_longptr(crdata->values+i*123+62+j), NULL));
      }
      add_item_to_menu(clevel, menu_new_item(0, 15, "Grow up   ", MIT_CHOICE, menu_value_longptr(crdata->values+i*123+41), CREATURES));
      add_item_to_menu(clevel, menu_new_item(0, 16, "Grow level", MIT_NUMERIC, menu_value_longptr(crdata->values+i*123+42), NULL));
      cstat->back=creatures;
      cannoy->back=creatures;
      clevel->back=creatures;
      cliving->back=creatures;
    }
    add_item_to_menu(root, menu_new_item(2, 2, "Creatures", MIT_MENU, menu_value_longptr(NULL), creatures));
    creatures->back=root;
    return creatures;
}

/*
 * Creates "Rooms" menu elements.
 */
menu init_rooms_menu(menu root,struct dk_creature_data *crdata)
{
    menu rooms;
    rooms = new_menu("Room costs and health values");
    add_item_to_menu(rooms, menu_new_item(2, 2, "Room", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(rooms, menu_new_item(22, 2, "Cost", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(rooms, menu_new_item(42, 2, "Health", MIT_STATIC, menu_value_longptr(NULL), NULL));
    int i;
    for (i=0; ROOMS[i+1]!=NULL; i++)
    {
      add_item_to_menu(rooms, menu_new_item(2, 4+i, ROOMS[i+1], MIT_STATIC, menu_value_longptr(NULL), NULL));
      add_item_to_menu(rooms, menu_new_item(21, 4+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+3814+i*3), NULL));
      add_item_to_menu(rooms, menu_new_item(41, 4+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+3815+i*3), NULL));
    }
    add_item_to_menu(root, menu_new_item(2, 3, "Rooms", MIT_MENU, menu_value_longptr(NULL), rooms));
    rooms->back=root;
    return rooms;
}

/*
 * Creates "Keeper powers" menu elements.
 */
menu init_powers_menu(menu root,struct dk_creature_data *crdata)
{
    menu powers1, powers2;
    char temp[LINEMSG_SIZE];
    powers1 = new_menu("Keeper spells (page 1)");
    add_item_to_menu(powers1, menu_new_item(0, 2, "Spell: Cost:", MIT_STATIC, menu_value_longptr(NULL), NULL));
    int i,j;
    for (i=0; i < 9; i++)
    {
      sprintf (temp, "%d", i);
      add_item_to_menu(powers1, menu_new_item(7*i+15, 2, strdup(temp), MIT_STATIC, menu_value_longptr(NULL), NULL));
    }
    for (i=0; POWERS[i]; i++)
    {
      add_item_to_menu(powers1, menu_new_item(0, 3+i, POWERS[i], MIT_STATIC, menu_value_longptr(NULL), NULL));
      for (j=0; j < 9; j++)
        add_item_to_menu(powers1, menu_new_item(7*j+14, 3+i, "", MIT_NUMERIC,
                       menu_value_longptr(crdata->values+3862+i*20+j), NULL));
    }

    powers2 = new_menu("Keeper spells (page 2)");
    add_item_to_menu(powers2, menu_new_item(0, 2, "Spell: Power:", MIT_STATIC, menu_value_longptr(NULL), NULL));
    for (i=0; i < 9; i++)
    {
      sprintf (temp, "%d", i);
      add_item_to_menu(powers2, menu_new_item(6*i+15, 2, strdup(temp), MIT_STATIC, menu_value_longptr(NULL), NULL));
    }
    add_item_to_menu(powers2, menu_new_item(69, 2, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    for (i=0; POWERS[i]; i++)
    {
      add_item_to_menu(powers2, menu_new_item(0, 3+i, POWERS[i], MIT_STATIC, menu_value_longptr(NULL), NULL));
      for (j=0; j < 9; j++)
        add_item_to_menu(powers2, menu_new_item(6*j+14, 3+i, "", MIT_NUMERIC, 
                       menu_value_longptr(crdata->values+3872+i*20+j), NULL));
      add_item_to_menu(powers2, menu_new_item(70, 3+i, "", MIT_NUMERIC,
          menu_value_longptr(crdata->values+3871+i*20), NULL));
    }
    add_item_to_menu(powers1, menu_new_item(0, 23, "Page 2", MIT_MENU, menu_value_longptr(NULL), powers2));
    add_item_to_menu(powers2, menu_new_item(0, 23, "Page 1", MIT_MENU, menu_value_longptr(NULL), powers1));

    add_item_to_menu(root, menu_new_item(2, 4, "Keeper powers", MIT_MENU, menu_value_longptr(NULL), powers1));
    powers1->back=root;
    powers2->back=root;
    return powers1;
}

/*
 * Creates "Miscellaneous" menu elements.
 */
menu init_misc_menu(menu root,struct dk_creature_data *crdata)
{
    menu misc1, misc2;
    misc1 = new_menu("Miscellaneous options (page 1)");
    add_item_to_menu(misc1, menu_new_item(0, 2, "Gold per block          ", MIT_NUMERIC, menu_value_longptr(crdata->values+4242), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 3, "Gold per pot            ", MIT_NUMERIC, menu_value_longptr(crdata->values+4244), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 4, "Gold pile value         ", MIT_NUMERIC, menu_value_longptr(crdata->values+4246), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 5, "Gold pile maximum       ", MIT_NUMERIC, menu_value_longptr(crdata->values+4248), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 6, "Recovery frequency      ", MIT_NUMERIC, menu_value_longptr(crdata->values+4250), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 7, "Fight maximum hate      ", MIT_NUMERIC, menu_value_longptr(crdata->values+4252), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 8, "Fight borderline        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4254), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 9, "Fight maximum love      ", MIT_NUMERIC, menu_value_longptr(crdata->values+4256), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 10, "Food life from hatchery ", MIT_NUMERIC, menu_value_longptr(crdata->values+4258), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 11, "Fight hate kill value   ", MIT_NUMERIC, menu_value_longptr(crdata->values+4260), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 12, "D/M statue reappear time", MIT_NUMERIC, menu_value_longptr(crdata->values+4262), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 13, "D/M object reappear time", MIT_NUMERIC, menu_value_longptr(crdata->values+4264), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 14, "Hits per slab           ", MIT_NUMERIC, menu_value_longptr(crdata->values+4266), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 15, "Collapse dungeon damage ", MIT_NUMERIC, menu_value_longptr(crdata->values+4268), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 16, "Turns per collapse      ", MIT_NUMERIC, menu_value_longptr(crdata->values+4270), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 17, "Ghost convert chance    ", MIT_NUMERIC, menu_value_longptr(crdata->values+4284), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 18, "Armoury time            ", MIT_NUMERIC, menu_value_longptr(crdata->values+4286), NULL));
    add_item_to_menu(misc1, menu_new_item(0, 19, "Workshop time           ", MIT_NUMERIC, menu_value_longptr(crdata->values+4288), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 2, "Observatory time          ", MIT_NUMERIC, menu_value_longptr(crdata->values+4290), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 3, "Observatory generate      ", MIT_NUMERIC, menu_value_longptr(crdata->values+4292), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 4, "Improve area              ", MIT_NUMERIC, menu_value_longptr(crdata->values+4294), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 5, "Door open for             ", MIT_NUMERIC, menu_value_longptr(crdata->values+4296), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 6, "Boulder reduce health slap", MIT_NUMERIC, menu_value_longptr(crdata->values+4298), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 7, "Boulder reduce health wall", MIT_NUMERIC, menu_value_longptr(crdata->values+4300), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 8, "Boulder reduce health room", MIT_NUMERIC, menu_value_longptr(crdata->values+4302), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 9, "Tile strength             ", MIT_NUMERIC, menu_value_longptr(crdata->values+4304), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 10, "Gold tile strength        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4306), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 11, "Armageddon count down     ", MIT_NUMERIC, menu_value_longptr(crdata->values+4308), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 12, "Armageddon duration       ", MIT_NUMERIC, menu_value_longptr(crdata->values+4310), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 13, "Minimum gold              ", MIT_NUMERIC, menu_value_longptr(crdata->values+4312), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 14, "Max gold lookup           ", MIT_NUMERIC, menu_value_longptr(crdata->values+4314), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 15, "Min gold to record        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4316), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 16, "Wait for room time        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4318), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 17, "Check expand time         ", MIT_NUMERIC, menu_value_longptr(crdata->values+4320), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 18, "Max distance to dig       ", MIT_NUMERIC, menu_value_longptr(crdata->values+4322), NULL));
    add_item_to_menu(misc1, menu_new_item(40, 19, "Wait after room area      ", MIT_NUMERIC, menu_value_longptr(crdata->values+4324), NULL));

    misc2 = new_menu("Miscellaneous options (page 2)");
    add_item_to_menu(misc2, menu_new_item(0, 2, "Per imp gold dig size           ", MIT_NUMERIC, menu_value_longptr(crdata->values+4326), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 3, "Default generate speed          ", MIT_NUMERIC, menu_value_longptr(crdata->values+4328), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 4, "Barrack time                    ", MIT_NUMERIC, menu_value_longptr(crdata->values+4330), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 5, "Food generation speed           ", MIT_NUMERIC, menu_value_longptr(crdata->values+4332), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 6, "Def. neutral entrance lvl       ", MIT_NUMERIC, menu_value_longptr(crdata->values+4334), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 7, "Def. max creatures gen entrance ", MIT_NUMERIC, menu_value_longptr(crdata->values+4336), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 8, "Def. imp dig damage             ", MIT_NUMERIC, menu_value_longptr(crdata->values+4338), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 9, "Def. imp dig own damage         ", MIT_NUMERIC, menu_value_longptr(crdata->values+4340), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 10, "Prison skeleton chance          ", MIT_NUMERIC, menu_value_longptr(crdata->values+4342), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 11, "Def. sacrifice score for horny  ", MIT_NUMERIC, menu_value_longptr(crdata->values+4344), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 12, "Game turns in flee              ", MIT_NUMERIC, menu_value_longptr(crdata->values+4346), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 13, "Train cost frequency            ", MIT_NUMERIC, menu_value_longptr(crdata->values+4348), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 14, "Torture convert chance          ", MIT_NUMERIC, menu_value_longptr(crdata->values+4350), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 15, "Pay day gap                     ", MIT_NUMERIC, menu_value_longptr(crdata->values+4352), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 16, "Chest gold hand                 ", MIT_NUMERIC, menu_value_longptr(crdata->values+4354), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 17, "Slab collapse time              ", MIT_NUMERIC, menu_value_longptr(crdata->values+4356), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 18, "Dungeon heart health            ", MIT_NUMERIC, menu_value_longptr(crdata->values+4358), NULL));
    add_item_to_menu(misc2, menu_new_item(0, 19, "H of E gold grab amount         ", MIT_NUMERIC, menu_value_longptr(crdata->values+4360), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 2, "Dungeon heart heal time        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4362), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 3, "Dungeon heart heal health      ", MIT_NUMERIC, menu_value_longptr(crdata->values+4364), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 4, "Hero door wait time            ", MIT_NUMERIC, menu_value_longptr(crdata->values+4366), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 5, "Disease transfer %             ", MIT_NUMERIC, menu_value_longptr(crdata->values+4368), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 6, "Disease lose % health          ", MIT_NUMERIC, menu_value_longptr(crdata->values+4370), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 7, "Disease lose health time       ", MIT_NUMERIC, menu_value_longptr(crdata->values+4372), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 8, "Hold audience time             ", MIT_NUMERIC, menu_value_longptr(crdata->values+4374), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 9, "A'g'don t/port your time gap   ", MIT_NUMERIC, menu_value_longptr(crdata->values+4376), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 10, "A'g'don t/port enemy time gap  ", MIT_NUMERIC, menu_value_longptr(crdata->values+4378), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 11, "Scavenge cost frequency        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4380), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 12, "Temple scavenge protection time", MIT_NUMERIC, menu_value_longptr(crdata->values+4382), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 13, "Bodies for vampire             ", MIT_NUMERIC, menu_value_longptr(crdata->values+4384), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 14, "Body remains for               ", MIT_NUMERIC, menu_value_longptr(crdata->values+4386), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 15, "Graveyard convert time         ", MIT_NUMERIC, menu_value_longptr(crdata->values+4388), NULL));
    add_item_to_menu(misc2, menu_new_item(40, 16, "Grave min dist for teleport    ", MIT_NUMERIC, menu_value_longptr(crdata->values+4390), NULL));
    
    add_item_to_menu(misc1, menu_new_item(0, 23, "Page 2", MIT_MENU, menu_value_longptr(NULL), misc2));
    add_item_to_menu(misc2, menu_new_item(0, 23, "Page 1", MIT_MENU, menu_value_longptr(NULL), misc1));

    add_item_to_menu(root, menu_new_item(2, 5, "Miscellaneous", MIT_MENU, menu_value_longptr(NULL), misc1));
    misc1->back=root;
    misc2->back=root;
    return misc1;
}
/*
 * Creates "Spell durations" menu elements.
 */
menu init_spelldur_menu(menu root,struct dk_creature_data *crdata)
{
    menu spell;
    spell = new_menu("Spell durations");
    add_item_to_menu(spell, menu_new_item(0, 2, "Spell", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(spell, menu_new_item(20, 2, "Duration", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(spell, menu_new_item(40, 2, "Spell", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(spell, menu_new_item(60, 2, "Duration", MIT_STATIC, menu_value_longptr(NULL), NULL));
    int i;
    for (i=0; i < 28; i++)
    {
    add_item_to_menu(spell, menu_new_item((i/14)*40, (i%14)+4, SPELLS[i], MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(spell, menu_new_item((i/14)*40+19, (i%14)+4, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4419+i*2), NULL));
    }
    add_item_to_menu(root, menu_new_item(2, 6, "Spell durations", MIT_MENU, menu_value_longptr(NULL), spell));
    spell->back=root;
    return spell;
}
/*
 * Creates "Shots" menu elements.
 */
menu init_shots_menu(menu root,struct dk_creature_data *crdata)
{
    menu shots;
    shots = new_menu("Keeper Shot data");
    add_item_to_menu(shots, menu_new_item(0, 2, "Description", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item(18, 2, "Health", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item(25, 2, "Damage", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item(32, 2, "Speed", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item(40, 2, "Description", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item(58, 2, "Health", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item(65, 2, "Damage", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item(72, 2, "Speed", MIT_STATIC, menu_value_longptr(NULL), NULL));
    int i;
    for (i=0; i < 29; i++)
    {
    add_item_to_menu(shots, menu_new_item((i/15)*40, 4+(i%15), SHOTS[i], MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(shots, menu_new_item((i/15)*40+17, 4+(i%15), "", MIT_NUMERIC, menu_value_longptr(crdata->values+4479+i*4), NULL));
    add_item_to_menu(shots, menu_new_item((i/15)*40+24, 4+(i%15), "", MIT_NUMERIC, menu_value_longptr(crdata->values+4480+i*4), NULL));
    add_item_to_menu(shots, menu_new_item((i/15)*40+31, 4+(i%15), "", MIT_NUMERIC, menu_value_longptr(crdata->values+4481+i*4), NULL));
    }
    add_item_to_menu(root, menu_new_item(2, 7, "Shots", MIT_MENU, menu_value_longptr(NULL), shots));
    shots->back=root;
    return shots;
}
/*
 * Creates "Traps and Doors" menu elements.
 */
menu init_trapdoor_menu(menu root,struct dk_creature_data *crdata)
{
    menu trap;
    trap = new_menu("Traps and Doors");
    add_item_to_menu(trap, menu_new_item(0, 2, "Description", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(20, 2, "Man. Lvl ", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(30, 2, "Man. Req", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(40, 2, "Shots", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(50, 2, "Reload", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(60, 2, "Resale value", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(20, 11, "Man. Lvl ", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(30, 11, "Man. Req", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(40, 11, "Resale", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(50, 11, "Health", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 4, "Boulder", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 5, "Alarm", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 6, "Poison gas", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 7, "Lightning", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 8, "Word of power", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 9, "Lava", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 13, "Wooden", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 14, "Braced", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 15, "Steel", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(trap, menu_new_item(0, 16, "Magic", MIT_STATIC, menu_value_longptr(NULL), NULL));
    int i;
    for (i=0; i < 6; i++)
    {
    add_item_to_menu(trap, menu_new_item(19, 4+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4600+i*6), NULL));
    add_item_to_menu(trap, menu_new_item(29, 4+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4602+i*6), NULL));
    add_item_to_menu(trap, menu_new_item(39, 4+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4603+i*6), NULL));
    add_item_to_menu(trap, menu_new_item(49, 4+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4604+i*6), NULL));
    add_item_to_menu(trap, menu_new_item(59, 4+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4605+i*6), NULL));
    }
    for (i=0; i < 4; i++)
    {
    add_item_to_menu(trap, menu_new_item(19, 13+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4637+i*5), NULL));
    add_item_to_menu(trap, menu_new_item(29, 13+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4638+i*5), NULL));
    add_item_to_menu(trap, menu_new_item(39, 13+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4639+i*5), NULL));
    add_item_to_menu(trap, menu_new_item(49, 13+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4640+i*5), NULL));
    }

    add_item_to_menu(root, menu_new_item(2, 8, "Traps and Doors", MIT_MENU, menu_value_longptr(NULL), trap));
    trap->back=root;
    return trap;
}
/*
 * Creates "Health / blocks" menu elements.
 */
menu init_health_menu(menu root,struct dk_creature_data *crdata)
{
    menu health;
    health = new_menu("Health and block tables");
    add_item_to_menu(health, menu_new_item(2, 2, "Health crdata->values", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(health, menu_new_item(2, 4, "Hunger health loss                ", MIT_NUMERIC, menu_value_longptr(crdata->values+4661), NULL));
    add_item_to_menu(health, menu_new_item(2, 5, "Game turns per hunger health loss ", MIT_NUMERIC, menu_value_longptr(crdata->values+4663), NULL));
    add_item_to_menu(health, menu_new_item(2, 6, "Food health gain                  ", MIT_NUMERIC, menu_value_longptr(crdata->values+4665), NULL));
    add_item_to_menu(health, menu_new_item(2, 7, "Torture health loss               ", MIT_NUMERIC, menu_value_longptr(crdata->values+4667), NULL));
    add_item_to_menu(health, menu_new_item(2, 8, "Game turns per torture health loss", MIT_NUMERIC, menu_value_longptr(crdata->values+4669), NULL));
    add_item_to_menu(health, menu_new_item(2, 10, "Room", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(health, menu_new_item(15, 10, "Health", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(health, menu_new_item(2, 12, "Rock        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4671), NULL));
    add_item_to_menu(health, menu_new_item(2, 13, "Gold        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4673), NULL));
    add_item_to_menu(health, menu_new_item(2, 14, "Path        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4675), NULL));
    add_item_to_menu(health, menu_new_item(2, 15, "Floor       ", MIT_NUMERIC, menu_value_longptr(crdata->values+4677), NULL));
    add_item_to_menu(health, menu_new_item(2, 16, "Room        ", MIT_NUMERIC, menu_value_longptr(crdata->values+4679), NULL));
    add_item_to_menu(health, menu_new_item(2, 17, "Wooden door ", MIT_NUMERIC, menu_value_longptr(crdata->values+4681), NULL));
    add_item_to_menu(health, menu_new_item(2, 18, "Braced door ", MIT_NUMERIC, menu_value_longptr(crdata->values+4683), NULL));
    add_item_to_menu(health, menu_new_item(2, 19, "Steel door  ", MIT_NUMERIC, menu_value_longptr(crdata->values+4685), NULL));
    add_item_to_menu(health, menu_new_item(2, 20, "Magic door  ", MIT_NUMERIC, menu_value_longptr(crdata->values+4687), NULL));

    add_item_to_menu(root, menu_new_item(2, 9, "Health / blocks", MIT_MENU, menu_value_longptr(NULL), health));
    health->back=root;
    return health;
}

/*
 * Creates "Instances" menu elements.
 */
menu init_instances_menu(menu root,struct dk_creature_data *crdata)
{
    menu instance1, instance2, instance3;
    instance1=new_menu("Instance data (page 1)");
    instance2=new_menu("Instance data (page 2)");
    instance3=new_menu("Instance data (page 3)");
    add_item_to_menu(instance1, menu_new_item(0, 3, "Name", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(15,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(24,2, "Action", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(24,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(33,2, "Reset", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(33,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(42,2, "FP", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(42,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(51,2, "FP Act", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(51,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(60,2, "FP Reset", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(60,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(69,2, "Force", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance1, menu_new_item(69,3, "Visibility", MIT_STATIC, menu_value_longptr(NULL), NULL));

    add_item_to_menu(instance2, menu_new_item(0, 3, "Name", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(15,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(24,2, "Action", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(24,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(33,2, "Reset", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(33,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(42,2, "FP", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(42,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(51,2, "FP Act", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(51,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(60,2, "FP Reset", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(60,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(69,2, "Force", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance2, menu_new_item(69,3, "Visibility", MIT_STATIC, menu_value_longptr(NULL), NULL));

    add_item_to_menu(instance3, menu_new_item(0, 3, "Name", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(15,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(24,2, "Action", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(24,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(33,2, "Reset", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(33,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(42,2, "FP", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(42,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(51,2, "FP Act", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(51,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(60,2, "FP Reset", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(60,3, "Time", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(69,2, "Force", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(instance3, menu_new_item(69,3, "Visibility", MIT_STATIC, menu_value_longptr(NULL), NULL));
    int i,j;
    for (i=0; i < 15; i++)
    {
      add_item_to_menu(instance1, menu_new_item(0, 5+i, INSTANCE[i+1], MIT_STATIC, menu_value_longptr(NULL), NULL));
      add_item_to_menu(instance2, menu_new_item(0, 5+i, INSTANCE[i+16], MIT_STATIC, menu_value_longptr(NULL), NULL));
      add_item_to_menu(instance3, menu_new_item(0, 5+i, INSTANCE[i+31], MIT_STATIC, menu_value_longptr(NULL), NULL));
    }
    
    for (i=0; i < 15; i++)
    {
      for (j=0; j < 7; j++)
      {
        add_item_to_menu(instance1, menu_new_item(14+9*j, 5+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4697+i*8+j), NULL));
        add_item_to_menu(instance2, menu_new_item(14+9*j, 5+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4817+i*8+j), NULL));
        add_item_to_menu(instance3, menu_new_item(14+9*j, 5+i, "", MIT_NUMERIC, menu_value_longptr(crdata->values+4937+i*8+j), NULL));
      }
    }
    
    add_item_to_menu(instance1, menu_new_item(0, 23, "Page 2", MIT_MENU, menu_value_longptr(NULL), instance2));
    add_item_to_menu(instance2, menu_new_item(0, 23, "Page 1", MIT_MENU, menu_value_longptr(NULL), instance1));
    add_item_to_menu(instance2, menu_new_item(8, 23, "Page 3", MIT_MENU, menu_value_longptr(NULL), instance3));
    add_item_to_menu(instance3, menu_new_item(0, 23, "Page 2", MIT_MENU, menu_value_longptr(NULL), instance2));
    add_item_to_menu(root, menu_new_item(2, 10, "Instances", MIT_MENU, menu_value_longptr(NULL), instance1));
    instance1->back=root;
    instance2->back=root;
    instance3->back=root;
    return instance1;
}

/*
 * Creates "Research" menu elements.
 */
menu init_research_menu(menu root,struct dk_creature_data *crdata)
{
    menu research;
    research=new_menu("Research models (?) and costs");
    add_item_to_menu(research, menu_new_item(0, 2, "Description", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(20, 2, "Model", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(30, 2, "Amount", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 2, "Description", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(60, 2, "Model", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(70, 2, "Amount", MIT_STATIC, menu_value_longptr(NULL), NULL));
    
    add_item_to_menu(research, menu_new_item(0, 4, "H of E", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 5, "Possess", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 6, "Slap", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 7, "Create imp", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 8, "Treasure room", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 9, "Hatchery", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 10, "Lair", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 11, "Research", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 12, "Training", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 13, "Sight of evil", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 14, "Bridge", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 15, "Speed", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 16, "Must obey", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 17, "Guard post", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 18, "Call to arms", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(0, 19, "Workshop", MIT_STATIC, menu_value_longptr(NULL), NULL));

    add_item_to_menu(research, menu_new_item(40, 4, "Conceal", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 5, "Barracks", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 6, "Hold audience", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 7, "Prison", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 8, "Cave in", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 9, "Torture", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 10, "Heal", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 11, "Temple", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 12, "Lightning", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 13, "Graveyard", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 14, "Protect", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 15, "Scavenger", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 16, "Chicken", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 17, "Disease", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 18, "Armageddon", MIT_STATIC, menu_value_longptr(NULL), NULL));
    add_item_to_menu(research, menu_new_item(40, 19, "Destroy walls", MIT_STATIC, menu_value_longptr(NULL), NULL));
    int i;
    for (i=0; i < 32; i++)
    {
      add_item_to_menu(research, menu_new_item((i>>4)*40+19, (i&15)+4, "", MIT_NUMERIC, menu_value_longptr(crdata->values+5079+i*2), NULL));
      add_item_to_menu(research, menu_new_item((i>>4)*40+29, (i&15)+4, "", MIT_NUMERIC, menu_value_longptr(crdata->values+5080+i*2), NULL));
    }
    add_item_to_menu(root, menu_new_item(2, 11, "Research", MIT_MENU, menu_value_longptr(NULL), research));
    research->back=root;
    return research;
}

/*
 * Creates the whole main menu of the program.
 */
menu init_menus(struct dk_creature_data *crdata)
{
    menu root, creatures, rooms, powers, misc, spell, shots;
    menu trap, health, instance, research;
    // Create root menu
    root=new_menu("Dungeon Keeper creatures.txt editor");
    root->back=NULL;
    // Create creatures menus... ouch!
    creatures=init_creatures_menu(root,crdata);
    // Create rooms menu
    rooms=init_rooms_menu(root,crdata);
    // Create keeper power menus
    powers=init_powers_menu(root,crdata);
    // Create miscellaneous menu
    misc=init_misc_menu(root,crdata);
    // Create spell menu
    spell=init_spelldur_menu(root,crdata);
    // Create shot menu
    shots=init_shots_menu(root,crdata);
    // Create traps and doors menu
    trap=init_trapdoor_menu(root,crdata);
    // Create health/block stats menu
    health=init_health_menu(root,crdata);
    // Create instances menu
    instance=init_instances_menu(root,crdata);
    // Create research menu
    research=init_research_menu(root,crdata);
    return root;
}
