/******************************************************************************/
// textmenu.c - Console-based menu interface module.
/******************************************************************************/
// Author:  Tomasz Lis
// Created: 31 May 2008

// Purpose:
//   Defines functions used to create menu on text screen

// Comment:
//   None.

//Copying and copyrights:
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
/******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>

#include "textmenu.h"

#include "globals.h"
#include "input_kb.h"
#include "output_scr.h"

extern short process_special(union _value val,void *param);

inline union _value menu_value_num(long num)
{ union _value ret; ret.num=num; return ret; }

inline union _value menu_value_longptr(long *longptr)
{ union _value ret; ret.longptr=longptr; return ret; }

/*
 * Allocates memory and fills new menu item
 */
menuitem menu_new_item(int x, int y, char *label, int type, union _value value,
          void *param)
{
    menuitem ret;
    ret = malloc (sizeof (struct _menuitem));
    ret->type=type;
    ret->x=x;
    ret->y=y;
    ret->label=label;
    ret->value=value;
    ret->param=param;
    if (type==MIT_BOOLEAN)
      menu_item_limits(ret,false,true);
    else
    if (type==MIT_NUMERIC)
      menu_item_limits(ret,DK_LONG_MIN,DK_LONG_MAX);
    else
      menu_item_limits(ret,0,DK_LONG_MAX);
    return ret;
}

void menu_item_limits(menuitem dest, long min, long max)
{
    dest->limit.min=min;
    dest->limit.max=max;
}

/*
 * Adds menuitem into menu
 */
void add_item_to_menu(menu dest, menuitem src)
{
    int i, j, x, y, p, ip, jp;
    int n;
    dest->nitems++;
    dest->items=realloc(dest->items, dest->nitems*sizeof(menuitem));
    if (!dest->items)
    die ("add_item_to_menu: Out of memory");
    dest->items[dest->nitems-1]=src;
    
    src->num=dest->nitems-1;
    src->up=-1;
    src->down=-1;
    src->left=-1;
    src->right=-1;

    if (src->type==MIT_STATIC)
      return;
    
    if ((dest->first==-1) || (src->y < dest->items[dest->first]->y) ||
      ((src->y == dest->items[dest->first]->y) && 
       (src->x < dest->items[dest->first]->x)))
    dest->first=src->num;
    
    if ((dest->last==-1) || (src->y > dest->items[dest->last]->y) ||
      ((src->y == dest->items[dest->last]->y) && 
       (src->x > dest->items[dest->last]->x)))
    dest->last=src->num;
    
    n=0;
    for (i=0; i < src->num; i++)
      if (dest->items[i]->type != MIT_STATIC)
        n++;
    if (n<1)
    {
      src->up=src->num;
      src->down=src->num;
      src->left=src->num;
      src->right=src->num;
    } else
    if (n==1)
    {
      for (i=0; i < src->num; i++)
      {
        if (dest->items[i]->type != MIT_STATIC)
          n=i;
      }
      src->up=n;
      src->down=n;
      src->left=n;
      src->right=n;
      dest->items[n]->up=src->num;
      dest->items[n]->down=src->num;
      dest->items[n]->left=src->num;
      dest->items[n]->right=src->num;
    } else
    {
      x = src->x;
      y = src->y;
      p = (x<<8)+y;
      for (i=0; src->up==-1 && i < src->num; i++)
      {
        if (dest->items[i]->type==MIT_STATIC)
          continue;
        ip = (dest->items[i]->x<<8)+dest->items[i]->y;
        if (ip==p)
        {
          //MESSAGE!! printf ("Item location: %d,%d\n", x, y);
          die ("add_item_to_menu: Two items at same place!");
        }
        j = dest->items[i]->up;
        if (j==-1)
          die ("add_item_to_menu: This shouldn't happen - no up");
        jp = (dest->items[j]->x<<8)+dest->items[j]->y;
        if ((jp < ip && p < ip && p > jp) ||
          ((jp > ip) && (p < ip || p > jp)))
        {
          src->up=j;
          src->down=i;
          dest->items[i]->up=src->num;
          dest->items[j]->down=src->num;
        }
      }
      p = (y<<8)+x;
      for (i=0; (src->left==-1) && (i<src->num); i++)
      {
        if (dest->items[i]->type==MIT_STATIC)
          continue;
        ip = (dest->items[i]->y<<8)+dest->items[i]->x;
        j = dest->items[i]->left;
        if (j==-1)
          die ("add_item_to_menu: This shouldn't happen - no left");
        jp = (dest->items[j]->y<<8)+dest->items[j]->x;
        if ((jp < ip && p < ip && p > jp) ||
          ((jp > ip) && (p < ip || p > jp)))
        {
          src->left=j;
          src->right=i;
          dest->items[i]->left=src->num;
          dest->items[j]->right=src->num;
        }
      }
    }
}

/*
 * Allocates memory and initializes new menu
 */
menu new_menu(char *title)
{
    menu ret;
    ret = malloc (sizeof (struct _menu));
    ret->nitems=0;
    ret->items=NULL;
    ret->back=NULL;
    ret->title=title;
    ret->first=-1;
    ret->last=-1;
    return ret;
}    

/*
 * Reacts on menu key events
 */
short action_menukey(struct workmenu *currmnu,int key)
{
    if (currmnu==NULL) return false;
    menu m;
    int it;
    m = currmnu->cmenu;
    if (m==NULL) return false;
    it = currmnu->citem;
    if ((it<0) || (it>=m->nitems))
    {  //Handling error situation
        currmnu->item_open=false;
        currmnu->cmenu = m->back;
        currmnu->citem = currmnu->cmenu->first;
        speaker_beep();
        return true;
    }

    if (currmnu->item_open)
    { // Key on opened item
      switch (m->items[it]->type)
      {
      case MIT_CHOICE:
          if (action_menukey_choice(currmnu,key))
            return true;
          break;
      case MIT_NUMERIC:
          if (action_menukey_numeric(currmnu,key))
            return true;
          break;
      default: // Bad state
          currmnu->item_open=false;
          break;
      }
    } else
    { // Key when no item is opened
      if (action_menukey_treemove(currmnu,key))
          return true;
    }
    return false;
}

/*
 * Reacts on menu numeric value key events
 */
short action_menukey_numeric(struct workmenu *currmnu,int key)
{
    menu m;
    int it;
    m = currmnu->cmenu;
    it = currmnu->citem;
    int len;
    len = strlen(currmnu->edit_value);
    switch (key)
    {
    case KEY_ESCAPE:
          currmnu->item_open=false;
          return true;
    case KEY_ENTER:
          if (len>0)
              *(m->items[it]->value.longptr)=atoi(currmnu->edit_value);
          currmnu->item_open=false;
          return true;
    case KEY_DEL:
    case KEY_BACKSP:
          if (len>0)
              currmnu->edit_value[len-1]='\0';
          return true;
/*
    case KEY_PGDOWN: // PgDown on opened item
          return true;
    case KEY_PGUP: // PgUp on opened item
          return true;
    case KEY_HOME:
          return true;
    case KEY_END:
          return true;
*/
    case KEY_NUM0:
    case KEY_NUM1:
    case KEY_NUM2:
    case KEY_NUM3:
    case KEY_NUM4:
    case KEY_NUM5:
    case KEY_NUM6:
    case KEY_NUM7:
    case KEY_NUM8:
    case KEY_NUM9:
        if (len<11)
        {
          currmnu->edit_value[len]=key;
          currmnu->edit_value[len+1]='\0';
          long val=atoi(currmnu->edit_value);
          if (val>currmnu->limit_max)
          {
            currmnu->edit_value[len]='\0';
            speaker_beep();
          }
        } else
        {
          speaker_beep();
        }
        return true;
    case KEY_NMINUS:
        if ((len==0)&&(currmnu->limit_min<0))
        {
          currmnu->edit_value[len]='-';
          currmnu->edit_value[len+1]='\0';
        }
        return true;
    }
    return false;
}

/*
 * Reacts on menu choice value key events
 */
short action_menukey_choice(struct workmenu *currmnu,int key)
{
    menu m;
    int it;
    m = currmnu->cmenu;
    it = currmnu->citem;
    switch (key)
    {
    case KEY_DEL:
    case KEY_BACKSP:
    case KEY_ESCAPE:
          currmnu->item_open=false;
          return true;
    case KEY_ENTER:
          (*m->items[it]->value.longptr)=currmnu->limit_min+currmnu->choicen;
          currmnu->item_open=false;
          return true;

    case KEY_UP:
        if (currmnu->choicen)
          currmnu->choicen--;
        else
          currmnu->choicen=currmnu->limit_max-currmnu->limit_min-1;
        return true;
    case KEY_DOWN:
        currmnu->choicen++;
        if (currmnu->choicen>=currmnu->limit_max-currmnu->limit_min)
          currmnu->choicen=0;
        return true;
    case KEY_RIGHT:
        currmnu->choicen+=10;
        if (currmnu->choicen >= currmnu->limit_max-currmnu->limit_min)
          currmnu->choicen-=(currmnu->limit_max-currmnu->limit_min);
        if (currmnu->choicen >= currmnu->limit_max-currmnu->limit_min)
          currmnu->choicen=0;
        return true;
    case KEY_LEFT:
        currmnu->choicen-=10;
        if (currmnu->choicen < 0)
          currmnu->choicen=(currmnu->limit_max-currmnu->limit_min)+currmnu->choicen;
        if ((currmnu->choicen<0) || (currmnu->choicen >= currmnu->limit_max-currmnu->limit_min))
          currmnu->choicen=currmnu->limit_max-currmnu->limit_min-1;
        return true;
    case KEY_PGDOWN: // PgDown on opened item
          currmnu->choicen+=10;
          if (currmnu->choicen >= currmnu->limit_max-currmnu->limit_min)
              currmnu->choicen=currmnu->limit_max-currmnu->limit_min-1;
          return true;
    case KEY_PGUP: // PgUp on opened item
          currmnu->choicen-=10;
          if (currmnu->choicen < 0)
              currmnu->choicen = 0;
          return true;
    case KEY_HOME:
          currmnu->choicen = 0;
          return true;
    case KEY_END:
          currmnu->choicen=currmnu->limit_max-currmnu->limit_min-1;
          return true;
    }
    return false;
}

/*
 * Reacts on menu tree movement key events
 */
short action_menukey_treemove(struct workmenu *currmnu,int key)
{
    menu m;
    int it;
    m = currmnu->cmenu;
    it = currmnu->citem;
    switch (key)
    {
    case KEY_UP:
        currmnu->citem=m->items[it]->up;
        return true;
    case KEY_DOWN:
        currmnu->citem=m->items[it]->down;
        return true;
    case KEY_RIGHT:
        currmnu->citem=m->items[it]->right;
        return true;
    case KEY_LEFT:
        currmnu->citem=m->items[it]->left;
        return true;
    case KEY_HOME:
        currmnu->citem=m->first;
        return true;
    case KEY_END:
        currmnu->citem=m->last;
        return true;
    case KEY_PGUP:
        {
          int i;
          for (i=0;i<6;i++)
          {
            currmnu->citem=m->items[it]->up;
            it = currmnu->citem;
          }
        };return true;
    case KEY_PGDOWN:
        {
          int i;
          for (i=0;i<6;i++)
          {
            currmnu->citem=m->items[it]->down;
            it = currmnu->citem;
          }
        };return true;
    case KEY_ESCAPE:
        if (m->back)
        {
          currmnu->cmenu = m->back;
          currmnu->citem = currmnu->cmenu->first;
        }
        return true;
    case KEY_ENTER:
        switch (m->items[it]->type)
        {
        case MIT_MENU:
          {
            menu param;
            param=(menu)m->items[it]->param;
            if (param==NULL) break;
            currmnu->cmenu = param;
            currmnu->citem = currmnu->cmenu->first;
          };break;
        case MIT_CHOICE:
          {
            currmnu->item_open=true;
            currmnu->choicen=0;
            char **c;
            c = (char **)m->items[it]->param;
            if (c==NULL) break;
            currmnu->limit_min=0;
            while (!c[currmnu->limit_min])
              currmnu->limit_min++;
            currmnu->limit_max=currmnu->limit_min;
            currmnu->choicew=0;
            while (c[currmnu->limit_max])
            {
              if (strlen(c[currmnu->limit_max]) > currmnu->choicew)
                currmnu->choicew=strlen(c[currmnu->limit_max]);
              currmnu->limit_max++;
            }
            currmnu->choicew++;
            currmnu->choicecols = (currmnu->limit_max-currmnu->limit_min+9)/10;
          };break;
        case MIT_NUMERIC:
            currmnu->item_open=true;
            currmnu->limit_min=m->items[it]->limit.min;
            currmnu->limit_max=m->items[it]->limit.max;
            sprintf (currmnu->edit_value, "%ld", *(m->items[it]->value.longptr));
            break;
        case MIT_BOOLEAN:
            *(m->items[it]->value.longptr)=1-*(m->items[it]->value.longptr);
            break;
        case MIT_SPECIAL:
            process_special(m->items[it]->value,m->items[it]->param);
            break;
        }
        return true;
    case KEY_DEL:
    case KEY_BACKSP:
        // del/backspace when no item is opened
        if (m->back)
        {
          currmnu->cmenu = m->back;
          currmnu->citem = currmnu->cmenu->first;
        }
        return true;
    }
    return false;
}

void draw_menuscr(struct workmenu *currmnu,unsigned int rows,unsigned int cols)
{
    if (currmnu==NULL) return;
    menu m;
    m = currmnu->cmenu;
    if (m==NULL) return;

    screen_setcolor(PRINT_COLOR_YELLOW_ON_BLACK);
    set_cursor_pos(0, (cols-(strlen(m->title)+2)>>1));
    screen_printf(" %s ",m->title);
    screen_setcolor(PRINT_COLOR_LGREY_ON_BLACK);
    int citem=currmnu->citem;

    int i;
    for (i=0; i < m->nitems; i++)
    {
      set_cursor_pos(m->items[i]->y, m->items[i]->x);
      if ((citem==i) && ((m->items[i]->type==MIT_MENU)
                   || (m->items[i]->type==MIT_SPECIAL)))
        screen_setcolor(PRINT_COLOR_YELLOW_ON_BLUE);
      screen_printf("%s",m->items[i]->label);
      screen_printchr(' ');
      if (citem==i)
        screen_setcolor(PRINT_COLOR_YELLOW_ON_BLUE);
      switch (m->items[i]->type)
      {
      case MIT_NUMERIC:
        if ((currmnu->item_open) && (citem == i))
          screen_printf("%s",currmnu->edit_value);
        else
          screen_printf("%ld", *(m->items[i]->value.longptr));
        break;
      case MIT_BOOLEAN:
        screen_printf("%s",*(m->items[i]->value.longptr) ? "Yes" : "No");
        break;
      case MIT_CHOICE:
        screen_printf("%s",((char **)m->items[i]->param)
                [*(m->items[i]->value.longptr)]);
        break;
      }
      if (citem==i)
        screen_setcolor(PRINT_COLOR_LGREY_ON_BLACK);
    }

    if ((citem<0) || (citem>=m->nitems)) return;
    menuitem it;
    it = m->items[citem];
    if ((currmnu->item_open) && (it->type==MIT_CHOICE))
    {
      int cr, cx, cy;
    
      screen_setcolor(PRINT_COLOR_YELLOW_ON_RED);
      cr = currmnu->limit_max-currmnu->limit_min > 10 ? 10 : currmnu->limit_max-currmnu->limit_min;
      cy = ((rows-cr)>>1);
      cx = ((cols-currmnu->choicecols*currmnu->choicew)>>1);
      SLsmg_fill_region (cy-1, cx-1, 
               cr+2, currmnu->choicew*currmnu->choicecols+2, ' ');
      for (i=0; i < currmnu->limit_max-currmnu->limit_min; i++)
      {
        set_cursor_pos((i%10)+cy, (i/10)*currmnu->choicew+cx);
        if (i==currmnu->choicen)
          screen_setcolor(PRINT_COLOR_YELLOW_ON_BLUE);
        else
          screen_setcolor(PRINT_COLOR_YELLOW_ON_RED);
        screen_printf("%s",((char **)it->param)[i+currmnu->limit_min]);
      }
    }

}

void draw_menu_cursor(struct workmenu *currmnu,unsigned int rows,unsigned int cols)
{
    if (currmnu==NULL) return;
    menu m;
    m = currmnu->cmenu;
    if (m==NULL) return;
    int citem=currmnu->citem;
    menuitem it;
    if ((citem<0) || (citem>=m->nitems))
      it = NULL;
    else
      it = m->items[citem];
    if ((currmnu->item_open) && (it!=NULL) && (it->type==MIT_NUMERIC))
    {
      set_cursor_pos(it->y, it->x+strlen(it->label)+1+strlen(currmnu->edit_value));
      set_cursor_visibility(true);
    } else
    {
      set_cursor_pos(rows-1, cols-1);
      set_cursor_visibility(false);
    }
}
